﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EXIFLOG
{
    /// <summary>
    /// The metadata of one image, as decoded by exiftool.exe,
    /// presented as a dictionary from labels to values.
    /// </summary>
    public class ImageMetadata : Dictionary<string, string>
    {
        /// <summary>
        /// Complete dump of the metadata, neatly formatted.
        /// </summary>
        public List<string> ToAllStrings()
        {
            var result = new List<string>(100);
            foreach (string key in this.Keys)
            {
                result.Add(key.PadLeft(40) + " : " + this[key]);
            }
            return result;
        }

        /// <summary>
        /// Selective dump of the metadata, neatly formatted.
        /// </summary>
        public List<string> ToSelectedStrings()
        {
            var result = new List<string>(100);
            foreach (string key in SelectedKeys)
            {
                string val;
                this.TryGetValue(key, out val);
                if (!String.IsNullOrEmpty(val)) result.Add(key.PadLeft(40) + " : " + val);
            }
            return result;
        }

        public static string[] SelectedKeys =
        {
            "File Name",
            "File Modification Date/Time",
            "Create Date",
            "Camera Model Name",
            "Model",              // alternative to Camera Model Name
            "Image Size",

            "GPS Latitude",
            "GPS Longitude",
            "GPS Altitude",

            "Frame Count",
            "Frame Rate",
            "Video Codec",
            "Compressor ID",
            "Bit Depth",

            "Exposure Time",
            "F Number",
            "ISO",
            "Focal Length",
            "Lens ID",
            "Lens Spec",

            "Long Exposure Noise Reduction",
            "Long Exposure Noise Reduction 2",
            "High ISO Noise Reduction",
            "Noise Reduction"
        };

        /// <summary>
        /// A one-line summary, suitable for logging.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            var sb = new StringBuilder();

            // All files have the first 2 fields, even if they are not images.

            // Date/time
            if (this.ContainsKey("Create Date"))
            {
                sb.Append(this["Create Date"].Substring(0,19).PadRight(22));
            }
            else
            {
                sb.Append(this["File Modification Date/Time"].Substring(0,19).PadRight(22));
            }
            sb.Append(" ");

            // File name
            sb.Append(this["File Name"].PadRight(10));
            
            // Exposure information
            if (this.ContainsKey("Exposure Time")) sb.Append(" " + (this["Exposure Time"] + "s").PadLeft(8));
            if (this.ContainsKey("ISO")) sb.Append(" ISO " + this["ISO"].PadLeft(5));

            // Focal length and f-number if known
            if (this.ContainsKey("F Number") && !this["F Number"].StartsWith("0"))
            {
                if (this.ContainsKey("Focal Length"))
                {
                    sb.Append(this["Focal Length"].Split(' ')[0].PadLeft(7) + "mm");
                }
                sb.Append(" f/" + this["F Number"].PadRight(4));
            }

            // Movie information if present
            if (this.ContainsKey("Frame Count")) sb.Append(this["Frame Count"].PadLeft(7) + " frames");
            if (this.ContainsKey("Frame Rate")) sb.Append(this["Frame Rate"].PadLeft(7) + " fps");

            if (this.ContainsKey("Video Codec"))
            {
                sb.Append("  Codec:" + this["Video Codec"]);
            }
            else if (this.ContainsKey("Compressor ID"))
            {
                sb.Append("  Codec:" + this["Compressor ID"]);
            }

            if (this.ContainsKey("Bit Depth")) sb.Append("  Bits:" + this["Bit Depth"]);

            // Temperature

            if (this.ContainsKey("Camera Temperature")) sb.Append("  Temp:" + this["Camera Temperature"].Replace(" ","").PadLeft(4));

            // Noise reduction

            string nr = "";
            if (this.ContainsKey("Long Exposure Noise Reduction"))
            {
                nr = this["Long Exposure Noise Reduction"];
            }
            if (nr == "Off" && this.ContainsKey("Long Exposure Noise Reduction 2") && this["Long Exposure Noise Reduction 2"] != "Off")
            {
                nr = this["Long Exposure Noise Reduction 2"];
            }
            if (nr == "" && this.ContainsKey("Noise Reduction"))
            {
                nr = this["Noise Reduction"];
            }
            if (nr != "") sb.Append("  NR:" + nr);

            // Image size

            if (this.ContainsKey("Image Size")) sb.Append("  " + this["Image Size"].PadLeft(9));

            // Camera

            string model = "";

            if (this.ContainsKey("Camera Model Name"))
            {
                model = this["Camera Model Name"];
            }
            else if (this.ContainsKey("Model"))
            {
                model = this["Model"];
            }

            string make = "";

            if (this.ContainsKey("Make"))
            {
                make = this["Make"].Split(' ')[0];    // don't go past first blank
            }

            if (!model.StartsWith(make, StringComparison.OrdinalIgnoreCase))
            {
                model = make + " " + model;
            }

            sb.Append("  " + model);

            //// Often incorrect
            //if (this.ContainsKey("Lens ID"))
            //{
            //    sb.Append("  " + this["Lens ID"]);
            //}

            // GPS information if present
            if (this.ContainsKey("GPS Latitude") && this.ContainsKey("GPS Longitude"))
            {
                string[] lat = this["GPS Latitude"].Split(' ');
                string[] lon = this["GPS Longitude"].Split(' ');
                sb.Append("  GPS:");
                sb.Append(lat[4] + lat[0] + "°" + lat[2] + lat[3]);   // Degree sign will be UTF-8
                sb.Append(lon[4] + lon[0] + "°" + lon[2] + lon[3]);
            }


            return sb.ToString();
        }

    }
}
